@@ -5,10 +5,10 @@ class HelpCommand(Command):
     name = 'help'
     help = 'Prints listing of commands.'
 
-    def execute(self, line):
+    def execute(self, line: str) -> None:
         print('Run `command -h/--help` for individual command usage.')
         for name, command in commands.items():
             if command == self:
                 continue
-            print('  %s: %s' % (name, command.help))
+            print(f'  {name}: {command.help}')
         print()
