@@ -1,8 +1,15 @@
 import os
 from typing import List, Tuple
 
-order = ['linux-x86.tbl', 'linux-x64.tbl', 'linux-x32.tbl', 'linux-arm.tbl', 'freebsd.tbl',
-         'linux-generic.tbl', 'linux-generic32.tbl']
+order = [
+    'linux-x86.tbl',
+    'linux-x64.tbl',
+    'linux-x32.tbl',
+    'linux-arm.tbl',
+    'freebsd.tbl',
+    'linux-generic.tbl',
+    'linux-generic32.tbl',
+]
 max_id = 0
 by_name = {}
 by_id: List[str] = []
