@@ -6,8 +6,7 @@ class Executor(GCCExecutor):
     flags = ['-std=c11']
     command_paths = ['gcc']
     ext = 'c'
-    name = 'C11'
-    test_program = '''
+    test_program = """
 #include <stdio.h>
 
 int main() {
@@ -16,4 +15,4 @@ int main() {
         putchar(ch);
     return 0;
 }
-'''
+"""
