@@ -5,8 +5,7 @@ class Executor(CPPExecutor):
     command = 'g++20'
     command_paths = ['g++-11', 'g++']
     std = 'c++20'
-    name = 'CPP20'
-    test_program = '''
+    test_program = """
 #include <iostream>
 
 int main() {
@@ -15,4 +14,4 @@ int main() {
     std::cout << input;
     return 0;
 }
-'''
+"""
