@@ -3,9 +3,8 @@ from dmoj.executors.asm_executor import GASExecutor, PlatformX86Mixin
 
 class Executor(PlatformX86Mixin, GASExecutor):
     as_name = 'as_x86'
-    name = 'GAS32'
 
-    test_program = r'''.intel_syntax noprefix
+    test_program = r""".intel_syntax noprefix
 
 .text
 .global	_start
@@ -35,4 +34,4 @@ _exit:
 .bss
 buffer:
 	.skip	4096
-'''  # noqa: W191
+"""  # noqa: W191
