@@ -5,11 +5,10 @@ from dmoj.executors.mono_executor import MonoExecutor
 
 class Executor(MonoExecutor):
     ext = 'vb'
-    name = 'MONOVB'
     command = 'mono-vbnc'
     compile_output_index = 0
 
-    test_program = '''\
+    test_program = """\
 Imports System
 
 Public Module modmain
@@ -17,7 +16,7 @@ Public Module modmain
      Console.WriteLine(Console.ReadLine())
    End Sub
 End Module
-'''
+"""
 
     def get_compile_args(self):
         return [self.get_command(), '/nologo', '/quiet', '/optimize+', '/out:%s' % self.get_compiled_file(), self._code]
