@@ -11,14 +11,14 @@ from dmoj.executors.mixins import NullStdoutMixin
 # and proceeds to run.
 class Executor(NullStdoutMixin, CompiledExecutor):
     ext = 'cl'
-    name = 'SBCL'
     command = 'sbcl'
     syscalls = ['personality']
     test_program = '(write-line (read-line))'
     address_grace = 262144
     data_grace = 262144
+    nproc = -1
 
-    compile_script = '''(compile-file "{code}")'''
+    compile_script = """(compile-file "{code}")"""
 
     def get_compile_args(self):
         return [self.get_command(), '--eval', self.compile_script.format(code=self._code), '--quit']
@@ -32,7 +32,7 @@ class Executor(NullStdoutMixin, CompiledExecutor):
             '--no-sysinit',
             '--no-userinit',
             '--load',
-            self.problem + ".fasl",
+            self.problem + '.fasl',
             '--quit',
             '--end-toplevel-options',
         ]
