@@ -1,21 +1,21 @@
 #!/usr/bin/python
 import logging
+import multiprocessing
 import os
 import signal
 import sys
-import multiprocessing
 import threading
 import traceback
 from enum import Enum
 from http.server import HTTPServer
 from itertools import groupby
 from operator import itemgetter
-from typing import Any, Callable, Dict, Generator, List, NamedTuple, Optional, Tuple, Union
+from typing import Any, Callable, Dict, Generator, List, NamedTuple, Optional, Set, Tuple, Union
 
 from dmoj import packet
 from dmoj.control import JudgeControlRequestHandler
 from dmoj.error import CompileError
-from dmoj.judgeenv import clear_problem_dirs_cache, env, get_supported_problems, startup_warnings
+from dmoj.judgeenv import clear_problem_dirs_cache, env, get_supported_problems_and_mtimes, startup_warnings
 from dmoj.monitor import Monitor
 from dmoj.problem import BatchedTestCase, Problem, TestCase
 from dmoj.result import Result
@@ -27,7 +27,7 @@ try:
     from setproctitle import setproctitle
 except ImportError:
 
-    def setproctitle(title):
+    def setproctitle(title: str) -> None:
         pass
 
 
@@ -99,7 +99,7 @@ class Judge:
 
             try:
                 clear_problem_dirs_cache()
-                self.packet_manager.supported_problems_packet(get_supported_problems())
+                self.packet_manager.supported_problems_packet(get_supported_problems_and_mtimes())
             except Exception:
                 log.exception('Failed to update problems.')
 
@@ -162,7 +162,7 @@ class Judge:
                     handler_func = ipc_handler_dispatch[ipc_type]
                 except KeyError:
                     raise RuntimeError(
-                        "judge got unexpected IPC message from worker: %s" % ((ipc_type, data),)
+                        'judge got unexpected IPC message from worker: %s' % ((ipc_type, data),)
                     ) from None
 
                 handler_func(report, *data)
@@ -209,10 +209,12 @@ class Judge:
         if is_sc:
             case_info = ''
         else:
-            case_info = '[%.3fs (%.3fs) | %dkb] %s%s' % (
+            case_info = '[%.3fs (%.3fs wall) | %dkb | %d switches (%d involuntary)] %s%s' % (
                 result.execution_time,
                 result.wall_clock_time,
                 result.max_memory,
+                sum(result.context_switches),
+                result.context_switches[1],
                 colored_feedback,
                 colored_aux_codes,
             )
@@ -222,7 +224,7 @@ class Judge:
 
     def _ipc_batch_begin(self, report, batch_number: int) -> None:
         self.packet_manager.batch_begin_packet()
-        report(ansi_style("#ansi[Batch #%d](yellow|bold)" % batch_number))
+        report(ansi_style('#ansi[Batch #%d](yellow|bold)' % batch_number))
 
     def _ipc_batch_end(self, _report, _batch_number: int) -> None:
         self.packet_manager.batch_end_packet()
@@ -232,7 +234,7 @@ class Judge:
         report(ansi_style('#ansi[Forcefully terminating grading. Temporary files may not be deleted.](red|bold)'))
 
     def _ipc_unhandled_exception(self, _report, message: str) -> None:
-        logger.error("Unhandled exception in worker process")
+        logger.error('Unhandled exception in worker process')
         self.log_internal_error(message=message)
 
     def abort_grading(self, submission_id: Optional[int] = None) -> None:
@@ -271,7 +273,7 @@ class Judge:
         if self.packet_manager:
             self.packet_manager.close()
 
-    def log_internal_error(self, exc: BaseException = None, message: str = None) -> None:
+    def log_internal_error(self, exc: Optional[BaseException] = None, message: Optional[str] = None) -> None:
         if not message:
             # If exc exists, raise it so that sys.exc_info() is populated with its data.
             if exc:
@@ -305,7 +307,7 @@ class JudgeWorker:
 
         self.worker_process_conn, child_conn = multiprocessing.Pipe()
         self.worker_process = multiprocessing.Process(
-            name="DMOJ Judge Handler for %s/%d" % (self.submission.problem_id, self.submission.id),
+            name='DMOJ Judge Handler for %s/%d' % (self.submission.problem_id, self.submission.id),
             target=self._worker_process_main,
             args=(child_conn, self.worker_process_conn),
         )
@@ -325,7 +327,7 @@ class JudgeWorker:
                 self.worker_process.kill()
                 raise
             except Exception:
-                logger.error("Failed to read IPC message from worker!")
+                logger.error('Failed to read IPC message from worker!')
                 raise
 
             if ipc_type == IPC.BYE:
@@ -343,7 +345,7 @@ class JudgeWorker:
                 logger.exception('Exception while waiting for worker to shut down, ignoring...')
             finally:
                 if self.worker_process.is_alive():
-                    logger.error("Worker is still alive, sending SIGKILL!")
+                    logger.error('Worker is still alive, sending SIGKILL!')
                     self.worker_process.kill()
 
     def request_abort_grading(self) -> None:
@@ -352,7 +354,7 @@ class JudgeWorker:
         try:
             self.worker_process_conn.send((IPC.REQUEST_ABORT, ()))
         except Exception:
-            logger.exception("Failed to send abort request to worker, did it race?")
+            logger.exception('Failed to send abort request to worker, did it race?')
 
     def _worker_process_main(
         self,
@@ -374,7 +376,7 @@ class JudgeWorker:
                 try:
                     ipc_type, data = judge_process_conn.recv()
                 except:  # noqa: E722, whatever happened, we have to abort now.
-                    logger.exception("Judge unexpectedly hung up!")
+                    logger.exception('Judge unexpectedly hung up!')
                     self._do_abort()
                     return
 
@@ -383,7 +385,7 @@ class JudgeWorker:
                 elif ipc_type == IPC.REQUEST_ABORT:
                     self._do_abort()
                 else:
-                    raise RuntimeError("worker got unexpected IPC message from judge: %s" % ((ipc_type, data),))
+                    raise RuntimeError('worker got unexpected IPC message from judge: %s' % ((ipc_type, data),))
 
         def _report_unhandled_exception() -> None:
             # We can't pickle the whole traceback object, so just send the formatted exception.
@@ -426,12 +428,17 @@ class JudgeWorker:
                 # this case, since we're blocking the main judge from proceeding.
                 ipc_recv_thread.join(timeout=IPC_TEARDOWN_TIMEOUT)
                 if ipc_recv_thread.is_alive():
-                    logger.error("Judge IPC recv thread is still alive after timeout, shutting worker down anyway!")
+                    logger.error('Judge IPC recv thread is still alive after timeout, shutting worker down anyway!')
 
             # FIXME(tbrindus): we need to do this because cleaning up temporary directories happens on __del__, which
             # won't get called if we exit the process right now (so we'd leak all files created by the grader). This
             # should be refactored to have an explicit `cleanup()` or similar, rather than relying on refcounting
             # working out.
+            from dmoj.executors.compiled_executor import _CompiledExecutorMeta
+
+            for cached_executor in _CompiledExecutorMeta.compiled_binary_cache.values():
+                cached_executor.is_cached = False
+                cached_executor.cleanup()
             self.grader = None
 
     def _grade_cases(self) -> Generator[Tuple[IPC, tuple], None, None]:
@@ -444,7 +451,7 @@ class JudgeWorker:
                 self, problem, self.submission.language, utf8bytes(self.submission.source)
             )
         except CompileError as compilation_error:
-            error = compilation_error.args[0] or b'compiler exited abnormally'
+            error = compilation_error.message or 'compiler exited abnormally'
             yield IPC.COMPILE_ERROR, (error,)
             return
         else:
@@ -452,25 +459,32 @@ class JudgeWorker:
             if hasattr(binary, 'warning') and binary.warning is not None:
                 yield IPC.COMPILE_MESSAGE, (binary.warning,)
 
-        yield IPC.GRADING_BEGIN, (self.grader.is_pretested,)
+        yield IPC.GRADING_BEGIN, (self.grader.run_pretests_only,)
 
         flattened_cases: List[Tuple[Optional[int], Union[TestCase, BatchedTestCase]]] = []
         batch_number = 0
+        batch_dependencies: List[Set[int]] = []
         for case in self.grader.cases():
             if isinstance(case, BatchedTestCase):
                 batch_number += 1
                 for batched_case in case.batched_cases:
                     flattened_cases.append((batch_number, batched_case))
+                batch_dependencies.append(set(case.dependencies))
             else:
                 flattened_cases.append((None, case))
 
         case_number = 0
         is_short_circuiting = False
         is_short_circuiting_enabled = self.submission.short_circuit
+        passed_batches: Set[int] = set()
         for batch_number, cases in groupby(flattened_cases, key=itemgetter(0)):
             if batch_number:
                 yield IPC.BATCH_BEGIN, (batch_number,)
 
+                dependencies = batch_dependencies[batch_number - 1]  # List is zero-indexed
+                if passed_batches & dependencies != dependencies:
+                    is_short_circuiting = True
+
             for _, case in cases:
                 case_number += 1
 
@@ -501,6 +515,9 @@ class JudgeWorker:
                 yield IPC.RESULT, (batch_number, case_number, result)
 
             if batch_number:
+                if not is_short_circuiting:
+                    passed_batches.add(batch_number)
+
                 yield IPC.BATCH_END, (batch_number,)
                 is_short_circuiting &= is_short_circuiting_enabled
 
@@ -531,10 +548,7 @@ def sanity_check():
 
         # However running as root on Linux is a Bad Idea
         if os.getuid() == 0:
-            print(
-                'running the judge as root is unsafe, please use an unprivileged user instead',
-                file=sys.stderr
-            )
+            print('running the judge as root is unsafe, please use an unprivileged user instead', file=sys.stderr)
             return False
 
         # Our sandbox filter is long but simple, so we can see large improvements
@@ -592,7 +606,9 @@ def main():  # pragma: no cover
         pass
 
     logging.basicConfig(
-        filename=logfile, level=logging.INFO, format='%(levelname)s %(asctime)s %(process)d %(module)s %(message)s'
+        filename=logfile,
+        level=judgeenv.log_level,
+        format='%(levelname)s %(asctime)s %(process)d %(module)s %(message)s',
     )
 
     setproctitle('DMOJ Judge %s on %s' % (env['id'], make_host_port(judgeenv)))
