@@ -1,6 +1,8 @@
 # type: ignore
+import sys
 
-INT_MAX_NUMBER_DIGITS = 10000
+# Drop to Python's limit, see https://github.com/python/cpython/issues/95778
+INT_MAX_NUMBER_DIGITS = getattr(sys.int_info, 'default_max_str_digits', 4300)
 int_ = int
 
 if isinstance(__builtins__, dict):
@@ -24,7 +26,7 @@ class patched_int_meta(type):
 
 
 class patched_int(int_, metaclass=patched_int_meta):
-    def __new__(cls, s, *args, **kwargs):
+    def __new__(cls, s=0, *args, **kwargs):
         if isinstance(s, str) and len(s) > INT_MAX_NUMBER_DIGITS:
             raise ValueError('integer is too long')
         if cls is patched_int:
